#include <iostream>
#include <jstick.hpp>
#include <windows.h>
#include <XInput.h>

#pragma comment(lib, "XInput.lib")


// TODO: Track device state of N devices
// TODO: Keep prev_state, so we can fire Events when buttons are pressed/releaed, etc.

#define MAX_DEVICES 4
#define SHORT_RANGE 32768.f;

bool device_exists[MAX_DEVICES];
XINPUT_STATE cur_state[MAX_DEVICES];
XINPUT_STATE prev_state[MAX_DEVICES];

bool jstick::Initialize() { return true; }

bool jstick::Cleanup() { return true; }

void jstick::JoystickServiceUpdate() { }

// XInput cannot tell us what type of device it is, so assume it's a legitimate Microsoft XB360 controller.
std::string jstick::GetDeviceName(int hwid) { return "Microsoft X-Box 360 pad"; }

jstick::ControllerType jstick::GetDeviceTypeFromName(const std::string &name) {
	// On Windows, we assume the standard controller, we have no way of knowing specifics currently.
	// ~josh.
	return ControllerType::XBox;
}

int jstick::NumJoysticksDetected() {
	// TODO: Check it properly!!
	if (JoystickDetected())
		return 1;
	return 0;
}




Vector2 jstick::GetDPadAxis(float deadzone, int hwid)
{
	// "Synthesize" a DPad vector based upon whether the buttons are pressed.
	// This vector will not be analog as is the case with the linux/jsdev implementation.
	Vector2 dpad;

	if (IsButtonDown(XBoxButton::DPadDown))
		dpad.y += SHORT_RANGE;
	if (IsButtonDown(XBoxButton::DPadRight))
		dpad.x += SHORT_RANGE;
	if (IsButtonDown(XBoxButton::DPadLeft))
		dpad.x -= SHORT_RANGE;
	if (IsButtonDown(XBoxButton::DPadUp))
		dpad.y -= SHORT_RANGE;

	return dpad;

}

/// XInput treats the DPad as buttons.
bool jstick::GetDPadIsAxisOrButtons() { return false; }

/// XInput does not work via an event-poll loop, so we do nothing here.
void jstick::ReadEventLoop()
{
	prev_state[0] = cur_state[0];


	DWORD dwUserIndex = (DWORD)0;

	XINPUT_STATE state;
	ZeroMemory(&state, sizeof(XINPUT_STATE));

	DWORD result = XInputGetState(dwUserIndex, &state);

	if (result == ERROR_SUCCESS)
	{
		cur_state[0] = state;
	}
}


void jstick::Vibrate(int LV, int RV, int hwid)
{

	XINPUT_VIBRATION vibration;
	ZeroMemory(&vibration, sizeof(XINPUT_VIBRATION));

	vibration.wLeftMotorSpeed = LV;
	vibration.wRightMotorSpeed = RV;

	XInputSetState(hwid, &vibration);
}

bool jstick::JoystickDetected(int hwid)
{
	DWORD dwUserIndex = (DWORD)hwid;

	XINPUT_STATE state;
	ZeroMemory(&state, sizeof(XINPUT_STATE));

	DWORD result = XInputGetState(dwUserIndex, &state);

	if (result == ERROR_SUCCESS) {
		return true;
	} else {
		std::cout << "Error Code: " << result << std::endl;
		return false;
	}
}


Vector2 jstick::GetLeftThumbstickAxis(float deadzone, int hwid)
{
	XINPUT_STATE state;
	ZeroMemory(&state, sizeof(XINPUT_STATE));

	DWORD result = XInputGetState(hwid, &state);

	if (result != ERROR_SUCCESS) {
		return {0, 0};
	}

	return Vector2(state.Gamepad.sThumbLX, state.Gamepad.sThumbLY);
}

short jstick::GetLeftTrigger(int hwid)
{
	XINPUT_STATE state;
	ZeroMemory(&state, sizeof(XINPUT_STATE));

	DWORD result = XInputGetState(hwid, &state);

	if (result != ERROR_SUCCESS) {
		return 0;
	}

	return state.Gamepad.bLeftTrigger;
}

short jstick::GetRightTrigger(int hwid)
{
	XINPUT_STATE state;
	ZeroMemory(&state, sizeof(XINPUT_STATE));

	DWORD result = XInputGetState(hwid, &state);

	if (result != ERROR_SUCCESS) {
		return 0;
	}

	return state.Gamepad.bRightTrigger;
}

Vector2 jstick::GetRightThumbstickAxis(float deadzone, int hwid)
{
	XINPUT_STATE state;
	ZeroMemory(&state, sizeof(XINPUT_STATE));

	DWORD result = XInputGetState(hwid, &state);

	if (result != ERROR_SUCCESS) {
		return {0, 0};
	}

	return Vector2(state.Gamepad.sThumbRX, state.Gamepad.sThumbRY);
}

int jstick::Connect(int hwid)
{
	return hwid;
}

bool jstick::Disconnect(int hwid) {
	return true;
}

bool jstick::Connected(int hwid) { return true; }

Vector2 jstick::GetRightThumbstickAxisNormalized(float deadzone, int hwid)
{
	Vector2 axisL = jstick::GetLeftThumbstickAxis(deadzone, hwid);
	if (axisL.Magnitude() > 0.f)
		return axisL.Normalized();
	return {0,0};
}

Vector2 jstick::GetLeftThumbstickAxisNormalized(float deadzone, int hwid)
{
	Vector2 axisR = jstick::GetRightThumbstickAxis(deadzone, hwid);


	return axisR.Normalized();
}



float jstick::GetLeftTriggerNormalized(int hwid) {
	return GetLeftTrigger(hwid) / SHORT_RANGE;
}

float jstick::GetRightTriggerNormalized(int hwid) {
	return GetRightTrigger(hwid) / SHORT_RANGE;
}


bool jstick::IsButtonDown(const XBoxButton& btn, int hwid)
{
	XINPUT_STATE state;
	ZeroMemory(&state, sizeof(XINPUT_STATE));

	DWORD result = XInputGetState(hwid, &state);

	if (result != ERROR_SUCCESS) {
		return false;
	}

	switch (btn) {
		case XBoxButton::Back:    if (state.Gamepad.wButtons & XINPUT_GAMEPAD_BACK)  { return true; } break;
		case XBoxButton::Start:   if (state.Gamepad.wButtons & XINPUT_GAMEPAD_START) { return true; } break;
		case XBoxButton::ThumbL:  if (state.Gamepad.wButtons & XINPUT_GAMEPAD_LEFT_THUMB)  { return true; } break;
		case XBoxButton::ThumbR:  if (state.Gamepad.wButtons & XINPUT_GAMEPAD_RIGHT_THUMB) { return true; } break;
		case XBoxButton::ButtonA: if (state.Gamepad.wButtons & XINPUT_GAMEPAD_A) { return true; } break;
		case XBoxButton::ButtonB: if (state.Gamepad.wButtons & XINPUT_GAMEPAD_B) { return true; } break;
		case XBoxButton::ButtonX: if (state.Gamepad.wButtons & XINPUT_GAMEPAD_X) { return true; } break;
		case XBoxButton::ButtonY: if (state.Gamepad.wButtons & XINPUT_GAMEPAD_Y) { return true; } break;

		case XBoxButton::BumperL:   if (state.Gamepad.wButtons & XINPUT_GAMEPAD_LEFT_SHOULDER) { return true; } break;
		case XBoxButton::BumperR:   if (state.Gamepad.wButtons & XINPUT_GAMEPAD_RIGHT_SHOULDER) { return true; } break;

		case XBoxButton::DPadUp:    if (state.Gamepad.wButtons & XINPUT_GAMEPAD_DPAD_UP) { return true; } break;
		case XBoxButton::DPadLeft:  if (state.Gamepad.wButtons & XINPUT_GAMEPAD_DPAD_LEFT) { return true; } break;
		case XBoxButton::DPadDown:  if (state.Gamepad.wButtons & XINPUT_GAMEPAD_DPAD_DOWN) { return true; } break;
		case XBoxButton::DPadRight: if (state.Gamepad.wButtons & XINPUT_GAMEPAD_DPAD_RIGHT) { return true; } break;
		default: return false;
	}

	return false;
}

